@@ -3,6 +3,7 @@
 import json
 from urllib import parse
 from typing import List
+from uuid import UUID
 
 from .apihelper import APIHelper
 
@@ -16,7 +17,7 @@ class Analyses():
       request_params = {}
       return self._get_analyses(request_params)
 
-   def get_by_name(self,analysis_name):
+   def get_by_name(self,analysis_name: str):
       params = {"name": parse.quote(analysis_name)}
       return self._get_analyses(params)
 
@@ -24,50 +25,50 @@ class Analyses():
       params = {"target_url": target_url}
       return self._get_analyses(params)
 
-   def get_by_search_term(self,search_term):
+   def get_by_search_term(self,search_term: str):
       params = {"search_term": parse.quote(search_term)}
       return self._get_analyses(params)
 
-   def get(self,guid):
+   def get(self,guid: UUID):
       uri = self.base_url + "/{}".format(guid)
       return APIHelper()._rest_request(uri,"GET")
 
-   def get_audits(self,guid):
+   def get_audits(self,guid: UUID):
       uri = self.base_url + "/{}/audits".format(guid)
       return APIHelper()._rest_paged_request(uri,"GET",'analysis_audits',{'page':0})
 
-   def create_scan(self,guid):
+   def create_scan(self,guid: UUID):
       uri = self.base_url + '/{}/scans'.format(guid)
       payload = {} #TODO add code for all the scan values
       return APIHelper()._rest_request(uri,"POST",json.dumps(payload))
 
-   def get_scans(self,guid):
+   def get_scans(self,guid: UUID):
       uri = self.base_url + "/{}/scans".format(guid)
       return APIHelper()._rest_paged_request(uri,"GET",'scans',{'page': 0})
 
-   def create(self,name,scans,business_unit_guid=None,email=None,owner=None):
+   def create(self,name: str,scans,business_unit_guid: UUID=None,email=None,owner: str=None):
       # basic create that adds only metadata. Use Scans().setup() to create a Scans object
       return self._create_or_update(method="CREATE",name=name,scans=scans,
-                  business_unit_guid=business_unit_guid,email=email,owner=owner,guid=None)
+                  business_unit_guid=business_unit_guid,email=email,owner=owner)
 
-   def update(self,guid,name,scans,business_unit_guid=None,email=None,owner=None):
+   def update(self,guid: UUID,name: str,scans,business_unit_guid: UUID=None,email=None,owner: str=None):
       return self._create_or_update(method="UPDATE",name=name,scans=scans,
                   business_unit_guid=business_unit_guid,email=email,owner=owner,guid=guid)
 
-   def get_scanner_variables(self,guid):
+   def get_scanner_variables(self,guid: UUID):
       uri = self.base_url + "/{}/scanner_variables".format(guid)
       return APIHelper()._rest_paged_request(uri,"GET", 'scanner_variables', {'page': 0})
 
-   def update_scanner_variable(self,analysis_guid,scanner_variable_guid,reference_key,value,description):
+   def update_scanner_variable(self,analysis_guid: UUID,scanner_variable_guid: UUID,reference_key: str,value: str,description: str):
       uri = self.base_url + '/{}/scanner_variables/{}'.format(analysis_guid,scanner_variable_guid)
       body = { 'reference_key': reference_key, 'value': value, 'description': description }
       return APIHelper()._rest_request(uri,"PUT",body)
 
-   def delete_scanner_variable(self,analysis_guid,scanner_variable_guid):
+   def delete_scanner_variable(self,analysis_guid: UUID,scanner_variable_guid: UUID):
       uri = self.base_url + '/{}/scanner_variables/{}'.format(analysis_guid,scanner_variable_guid)
       return APIHelper()._rest_request(uri,'DELETE')
 
-   def delete(self,guid):
+   def delete(self,guid: UUID):
       uri = self.base_url + "/{}".format(guid)
       return APIHelper()._rest_request(uri,"DELETE")
 
@@ -76,7 +77,7 @@ class Analyses():
    def _get_analyses(self,params):
       return APIHelper()._rest_paged_request(self.base_url,"GET","analyses",params=params)
 
-   def _create_or_update(self,method,name,scans,business_unit_guid=None,email=None,owner=None,guid=None):
+   def _create_or_update(self,method,name: str,scans,business_unit_guid: UUID=None,email=None,owner: str=None,guid: UUID=None):
       if method == 'CREATE':
          uri = self.base_url
          httpmethod = 'POST'
@@ -102,37 +103,37 @@ class Analyses():
 class Scans():
    base_url = ROOT_URL + '/scans'
 
-   def get(self, guid):
+   def get(self, guid: UUID):
       uri = self.base_url + "/{}".format(guid)
       return APIHelper()._rest_request(uri,"GET")
 
-   def get_audits(self, guid):
+   def get_audits(self, guid: UUID):
       uri = self.base_url + "/{}/audits".format(guid)
       return APIHelper()._rest_paged_request(uri,"GET",'scan_audits',{'page':0})
 
-   def get_configuration(self,guid):
+   def get_configuration(self,guid: UUID):
       uri = self.base_url + "/{}/configuration".format(guid)
       return APIHelper()._rest_request(uri,"GET")
 
-   def delete(self,guid):
+   def delete(self,guid: UUID):
       uri = self.base_url + '/{}'.format(guid)
       return APIHelper()._rest_request(uri, 'DELETE')
 
-   def update(self,guid,scan):
+   def update(self,guid: UUID,scan):
       # use DynUtils().setup_scan() to create the scans parameter
       uri = self.base_url + '/{}'.format(guid)
       return APIHelper()._rest_request(uri,'PUT',body=json.dumps(scan))
 
-   def get_scanner_variables(self,guid):
+   def get_scanner_variables(self,guid: UUID):
       uri = self.base_url + "/{}/scanner_variables".format(guid)
       return APIHelper()._rest_paged_request(uri,"GET",'scanner_variables',{'page': 0})
 
-   def update_scanner_variable(self,scan_guid,scanner_variable_guid,reference_key,value,description):
+   def update_scanner_variable(self,scan_guid: UUID,scanner_variable_guid: UUID,reference_key: str,value: str,description: str):
       uri = self.base_url + '/{}/scanner_variables/{}'.format(scan_guid,scanner_variable_guid)
       body = { 'reference_key': reference_key, 'value': value, 'description': description }
       return APIHelper()._rest_request(uri,"PUT",body)
 
-   def delete_scanner_variable(self,scan_guid,scanner_variable_guid):
+   def delete_scanner_variable(self,scan_guid: UUID,scanner_variable_guid: UUID):
       uri = self.base_url + '/{}/scanner_variables/{}'.format(scan_guid,scanner_variable_guid)
       return APIHelper()._rest_request(uri,'DELETE')   
 
@@ -142,46 +143,46 @@ class Occurrences():
    def get_all(self):
       return APIHelper()._rest_paged_request(self.base_url,'GET','analysis_occurrences',{'page':0})
 
-   def get(self,guid):
+   def get(self,guid: UUID):
       uri = self.base_url + '/{}'.format(guid)
       return APIHelper()._rest_request(uri, 'GET')
 
-   def stop(self, guid, save_or_delete):
+   def stop(self, guid: UUID, save_or_delete):
       actions = {'SAVE': 'STOP_SAVE', 'DELETE': 'STOP_DELETE'}
       uri = self.base_url + '/{}'.format(guid)
       params = { 'action' : actions[save_or_delete] }
       return APIHelper()._rest_request(uri, 'PUT', json.dumps(params))
 
-   def get_scan_occurrences(self,guid):
-      uri = self.base_url + '/{}/scan_occurrences'
+   def get_scan_occurrences(self,guid: UUID):
+      uri = self.base_url + '/{}/scan_occurrences'.format(guid)
       return APIHelper()._rest_paged_request(uri, 'GET', 'scan_occurrences',{'page':0})
 
 class ScanOccurrences():
    base_url = ROOT_URL + '/scan_occurrences'
 
-   def get(self, guid):
+   def get(self, guid: UUID):
       uri = self.base_url + '/{}'.format(guid)
       return APIHelper()._rest_request(uri,'GET')
 
-   def stop(self,guid,save_or_delete):
+   def stop(self,guid: UUID,save_or_delete):
       actions = {'SAVE': 'STOP_SAVE', 'DELETE': 'STOP_DELETE'}
       uri = self.base_url + '/{}'.format(guid)
       params = { 'action': actions[save_or_delete]}
       return APIHelper()._rest_request(uri,'PUT', json.dumps(params))
 
-   def get_configuration(self,guid):
+   def get_configuration(self,guid: UUID):
       uri = self.base_url + '/{}/configuration'.format(guid)
       return APIHelper()._rest_request(uri,'GET')
    
-   def get_verification_report(self,guid):
+   def get_verification_report(self,guid: UUID):
       uri = self.base_url + '/{}/verification_report'.format(guid)
       return APIHelper()._rest_request(uri,'GET')
 
-   def get_scan_notes_report(self,guid):
+   def get_scan_notes_report(self,guid: UUID):
       uri = self.base_url + '/{}/scan_notes_report'.format(guid)
       return APIHelper()._rest_request(uri,'GET')
 
-   def get_scan_screenshots(self,guid):
+   def get_scan_screenshots(self,guid: UUID):
       uri = self.base_url + '/{}/scan_screenshots'.format(guid)
       return APIHelper()._rest_request(uri,'GET')
 
@@ -191,7 +192,7 @@ class CodeGroups():
    def get_all(self):
       return APIHelper()._rest_request(self.base_url,'GET')
 
-   def get(self,name):
+   def get(self,name: str):
       uri = self.base_url + '/{}'.format(name)
       return APIHelper()._rest_request(uri,'GET')
 
@@ -207,20 +208,20 @@ class ScannerVariables():
    def get_all(self):
       return APIHelper()._rest_paged_request(self.base_url,"GET",'scanner_variables',{'page': 0})
 
-   def create(self,reference_key,value,description):
+   def create(self,reference_key: str,value: str,description: str):
       payload = {'reference_key':reference_key, 'value': value, 'description': description}
       return APIHelper()._rest_request(self.base_url,'POST',json.dumps(payload))
 
-   def get(self,guid):
+   def get(self,guid: UUID):
       uri = self.base_url + '/{}'.format(guid)
       return APIHelper()._rest_request(uri,"GET")
 
-   def update(self,guid,reference_key,value,description):
+   def update(self,guid: UUID,reference_key: str,value: str,description: str):
       uri = self.base_url + '/{}'.format(guid)
       body = { 'reference_key': reference_key, 'value': value, 'description': description }
       return APIHelper()._rest_request(uri,"PUT",body)
 
-   def delete(self,guid):
+   def delete(self,guid: UUID):
       uri = self.base_url + '/{}'.format(guid)
       return APIHelper()._rest_request(uri,'DELETE')   
 
@@ -231,7 +232,7 @@ class ScanCapacitySummary():
 
 class DynUtils():
 
-   def setup_user_agent(self,custom_header,type):
+   def setup_user_agent(self,custom_header: str,type):
       return { "custom_header": custom_header, "type": type}
 
    def setup_custom_host(self,host_name,ip_address):
@@ -244,7 +245,7 @@ class DynUtils():
       # use to format any URL being made
       return { 'url': url, 'directory_restriction_type': directory_restriction_type, 'http_and_https': http_and_https}
 
-   def setup_scan_setting(self,blocklist_configs:list,custom_hosts:List, user_agent:None):
+   def setup_scan_setting(self,blocklist_configs:list,custom_hosts:List, user_agent: str=None):
       payload = {}
       
       if len(blocklist_configs) > 0:
@@ -258,19 +259,19 @@ class DynUtils():
 
       return { 'scan_setting': payload }
 
-   def setup_scan_contact_info(self,email,first_and_last_name,telephone):
+   def setup_scan_contact_info(self,email,first_and_last_name: str,telephone):
       return {'scan_contact_info': {'email': email, 'first_and_last_name': first_and_last_name, 'telephone': telephone}}
 
-   def setup_crawl_script(self,script_body,script_type='SELENIUM'):
+   def setup_crawl_script(self,script_body: str,script_type='SELENIUM'):
       return { 'crawl_script_data': { 'script_body': script_body, 'script_type': script_type}}
 
    def setup_crawl_configuration(self,scripts:List,disabled=False):
       return { 'crawl_configuration': { 'disabled': disabled, 'scripts': scripts}}
 
-   def setup_login_logout_script(self,script_body,script_type='SELENIUM'):
+   def setup_login_logout_script(self,script_body: str,script_type='SELENIUM'):
       return { 'script_body': script_body, 'script_type': script_type}
 
-   def setup_auth(self,authtype,username,password,domain=None,base64_pkcs12=None,cert_name=None, login_script_data=None, logout_script_data=None):
+   def setup_auth(self,authtype,username: str,password: str,domain=None,base64_pkcs12=None,cert_name: str=None, login_script_data: str=None, logout_script_data: str=None):
       payload = {}
       if authtype == 'AUTO':
          payload.update({'AUTO': {'authtype': authtype, 'username': username, 'password': password}})
@@ -298,7 +299,7 @@ class DynUtils():
          payload.update(crawl_config)
       return { 'scan_config_request': payload }
 
-   def setup_scan(self, scan_config_request, scan_contact_info=None, linked_app_guid=None):
+   def setup_scan(self, scan_config_request, scan_contact_info=None, linked_app_guid: UUID=None):
       payload = {}
       payload.update( scan_config_request )
       payload.update(scan_contact_info)
